﻿using System;
using System.Collections.Generic;
using System.Net;
using System.Xml.Linq;

namespace AutocompleteBox.WPF
{
	public class GoogleAutocompleteProvider : IAutocompleteProvider
	{
		WebClient client;

		public GoogleAutocompleteProvider()
		{
			this.client = new WebClient();
		}

		public event SuggestionsReceivedEventHandler SuggestionsReceived;

		public void GetSuggestions(string text)
		{
			this.client.CancelAsync();

			client.DownloadStringCompleted += this.ClientDownloadStringCompleted;
			client.DownloadStringAsync(new Uri("http://www.google.com/complete/search?hl=en-us&client=ie8&q=" + text, UriKind.Absolute));
		}

		private void ClientDownloadStringCompleted(object sender, DownloadStringCompletedEventArgs e)
		{
			if (!e.Cancelled)
			{
				this.OnSuggestionsReceived(this.ParseResult(e.Result));
			}
		}

		private IList<string> ParseResult(string result)
		{
			var xml = XDocument.Parse((string)result);

			var values = new List<string>();
			foreach (var node in xml.Descendants(XName.Get("Text", "http://opensearch.org/searchsuggest2")))
			{
				values.Add(node.Value);
			}

			return values;
		}

		protected virtual void OnSuggestionsReceived(IList<string> results)
		{
			if (this.SuggestionsReceived != null)
			{
				this.SuggestionsReceived(this, new SuggestionsReceivedEventArgs(results));
			}
		}
	}
}